<?php
if (!defined('_PS_VERSION_'))
    exit;

class PAPintegration extends Module {
    public $debug = false;

    function __construct() {
        $this->name = 'papintegration';
        $this->tab = 'analytics_stats';
        $this->version = '3.20.1';
        $this->author = 'Quality Unit';
        $this->displayName = 'Post Affiliate Pro integration';

        parent::__construct();

        if ($this->id && !Configuration::get('PAP_INSTALL_URL')
                && !Configuration::get('PAP_MERCHANT_USERNAME')
                && !Configuration::get('PAP_MERCHANT_PASSWORD')) {
            $this->warning = $this->l('You have not set your installation URL yet');
        }

        $this->description = $this->l('Integrate Post Affiliate Pro or Post Affiliate Network with your shop');
        $this->confirmUninstall = $this->l('Are you sure you want to DELETE your details?');

        /* Backward compatibility */
        require(_PS_MODULE_DIR_.$this->name.'/backward_compatibility/backward.php');
    }

    function debug($message) {
        if (!$this->debug) return;

        $f = fopen(_PS_MODULE_DIR_.$this->name.'/log.txt', 'a');
        fwrite($f, date("[Y-m-d H:i:s] ").$message."\n");
        fclose($f);
    }

    function install() {
        $usedHooks = array(
          'displayCustomerAccountForm',
          'actionCustomerAccountAdd',
          'actionOrderStatusUpdate'
        );
        if (version_compare(_PS_VERSION_, '1.7.0', '>=') === true){
            array_push($usedHooks, 'displayFooter', 'displayFooterBuilder', 'displayOrderConfirmation');
        } else {
            array_push($usedHooks, 'footer', 'orderConfirmation');
        }

        if (!parent::install()) {
            $this->debug('A problem with installation occurred.');
            return false;
        }

        foreach ($usedHooks as $hook) {
          if (!$this->registerHook($hook)) {
              $this->debug("A problem with installation occurred, registration of hook $hook failed.");
              return false;
          }
        }

        return true;
    }

  function uninstall() {
      if (!Configuration::deleteByName('PAP_INSTALL_URL')
          || !Configuration::deleteByName('PAP_ACCOUNT_ID')
          || !Configuration::deleteByName('PER_PRODUCT')
          || !Configuration::deleteByName('REGISTER_AFFILIATE')
          || !Configuration::deleteByName('AFFILIATE_APPROVAL_PRODUCT_ID')
          || !Configuration::deleteByName('PAP_MERCHANT_USERNAME')
          || !Configuration::deleteByName('PAP_MERCHANT_PASSWORD')
          || !Configuration::deleteByName('PAP_APPROVAL_ID')
          || !Configuration::deleteByName('PAP_DECLINE_ID')
          || !Configuration::deleteByName('PAP_CAMPAIGN_ID')
          || !parent::uninstall()) {
        $this->debug('A problem with uninstallation occurred.');
        return false;
      }

      return true;
  }

    private function getSession() {
        $pap_install_url = Configuration::get('PAP_INSTALL_URL');
        $username = Configuration::get('PAP_MERCHANT_USERNAME');
        $pass = Configuration::get('PAP_MERCHANT_PASSWORD');
        
        $response = array('session' => null, 'message' => '');

        $session = new Pap_Api_Session('http://'.$pap_install_url.'scripts/server.php');
        if (!@$session->login($username, $pass)) {
            // try HTTPS as well
            $session = new Pap_Api_Session('https://'.$pap_install_url.'scripts/server.php');
            if (!@$session->login($username, $pass)) {
                $response['message'] = $this->l('The connection to your Post Affiliate Pro failed!'). ' Error: '.$session->getMessage();
                return null;
            } else {
                $response['message'] = $this->l('The connection to your Post Affiliate Pro worked correctly.');
            }
        }
        else {
            $response['message'] = $this->l('The connection to your Post Affiliate Pro worked correctly.');
        }
        
        $response['session'] = $session;
        return $response;
    }

  public function getContent() {
      $output = '<h2>Post Affiliate Pro integration</h2>';
      if (Tools::isSubmit('submitPAP')) {
          if (($pap_url = Tools::getValue('pap_install_url')) &&
                  ($username = Tools::getValue('pap_merchant_username')) &&
                  ($pass = Tools::getValue('pap_merchant_password'))) {
              if (($account = Tools::getValue('pap_account_id'))) {
                  Configuration::updateValue('PAP_ACCOUNT_ID', $account);
              }
              else {
                  Configuration::updateValue('PAP_ACCOUNT_ID', 'default1');
              }

              $pap_url = str_replace('http://','',$pap_url);
              $pap_url = str_replace('https://','',$pap_url);
              if (strrpos($pap_url,'/') != (strlen($pap_url)-1)) {
                  $pap_url .= '/'; // add trailing slash
              }

              Configuration::updateValue('PAP_INSTALL_URL', $pap_url);
              Configuration::updateValue('PAP_MERCHANT_USERNAME', $username);
              Configuration::updateValue('PAP_MERCHANT_PASSWORD', $pass);
              Configuration::updateValue('PAP_APPROVAL_ID', Tools::getValue('pap_approval_id'));
              Configuration::updateValue('PAP_DECLINE_ID', Tools::getValue('pap_decline_id'));
              Configuration::updateValue('PAP_CAMPAIGN_ID', Tools::getValue('pap_campaign_id'));

              Configuration::updateValue('EXTRA_DATA1', Tools::getValue('extra_data1'));
              Configuration::updateValue('EXTRA_DATA2', Tools::getValue('extra_data2'));
              Configuration::updateValue('EXTRA_DATA3', Tools::getValue('extra_data3'));
              Configuration::updateValue('EXTRA_DATA4', Tools::getValue('extra_data4'));
              Configuration::updateValue('EXTRA_DATA5', Tools::getValue('extra_data5'));

              // per product tracking
              Configuration::updateValue('PER_PRODUCT', Tools::getValue('per_product'));

              // register affiliate
              $register = Tools::getValue('register_affiliate');
              if (!isset($register) || ($register != 'Y')) {
                  Configuration::updateValue('REGISTER_AFFILIATE', false);
                  Configuration::updateValue('AFFILIATE_APPROVAL_PRODUCT_ID', '');
              }
              else {
                  Configuration::updateValue('REGISTER_AFFILIATE', true);
                  Configuration::updateValue('AFFILIATE_APPROVAL_PRODUCT_ID', Tools::getValue('affiliate_approval_prodid'));
              }

              $output .= $this->displayConfirmation($this->l('Settings updated'));

              // Test PAP connection
              if (file_exists(_PS_MODULE_DIR_.$this->name.'/PapApi.class.php')) {
                  require_once (_PS_MODULE_DIR_.$this->name.'/PapApi.class.php');
                  //$pap_install_url = Configuration::get('PAP_INSTALL_URL');
                  
                  $response = $this->getSession();
                  if ($response['session'] === null) {
                      $output .= $this->displayError($response['message']);
                  } else {
                      $output .= $this->displayConfirmation($response['message']);
                  }
              }
              else {
                  $output .= $this->displayError($this->l('The API file is missing! Please download the API file and upload it to your "papintegration" module directory.'));
              }
          }
          else {
              $output .= $this->displayError($this->l('Please, define an installation URL, a merchant username and a password first!'));
          }
      }
      return $output.$this->displayForm();
  }

    public function displayForm() {
        $output = '<form action="'.Tools::safeOutput($_SERVER['REQUEST_URI']).'" method="post">
        <fieldset>
          <legend>'.$this->l('General settings').'</legend>

          <label>'.$this->l('Installation URL').'</label>
          <div class="margin-form">
            <input type="text" name="pap_install_url" value="'.Tools::safeOutput(Tools::getValue('pap_install_url', Configuration::get('PAP_INSTALL_URL'))).'" />
            <p class="clear">'.$this->l('Example').': www.yoursite.com/pap/ or account.postaffiliatepro.com/</p>
          </div>

          <label>'.$this->l('Merchant username').'</label>
          <div class="margin-form">
            <input type="text" name="pap_merchant_username" value="'.Tools::safeOutput(Tools::getValue('pap_merchant_username', Configuration::get('PAP_MERCHANT_USERNAME'))).'" />
          </div>

          <label>'.$this->l('Merchant password').'</label>
          <div class="margin-form">
            <input type="password" name="pap_merchant_password" value="'.Tools::safeOutput(Tools::getValue('pap_merchant_password', Configuration::get('PAP_MERCHANT_PASSWORD'))).'" />
          </div>

          <label>'.$this->l('Account ID').'</label>
          <div class="margin-form">
            <input type="text" name="pap_account_id" value="'.Tools::safeOutput(Tools::getValue('pap_account_id', Configuration::get('PAP_ACCOUNT_ID'))).'" />
            <p class="clear">('.$this->l('applies only for Post Affiliate Network').')</p>
          </div>
        </fieldset>

        <fieldset>
          <legend>'.$this->l('Tracking settings').'</legend>

          <label>'.$this->l('Per product tracking').'</label>
          <div class="margin-form">
            <select name="per_product" style="color:#000;width:145px">
              <option value="0" '.((Tools::getValue('per_product', Configuration::get('PER_PRODUCT')) == '0')?'selected':'').'>per order instead</option>
              <option value="1" '.((Tools::getValue('per_product', Configuration::get('PER_PRODUCT')) == '1')?'selected':'').'>use product ID</option>
              <option value="2" '.((Tools::getValue('per_product', Configuration::get('PER_PRODUCT')) == '2')?'selected':'').'>use product name</option>
              <option value="3" '.((Tools::getValue('per_product', Configuration::get('PER_PRODUCT')) == '3')?'selected':'').'>use category ID</option>
            </select>
            <p class="clear">'.$this->l('With per product tracking, every item from each order will generate a separate commission').'</p>
          </div>

          <label>'.$this->l('Force campaign').'</label>
          <div class="margin-form">
            <input type="text" name="pap_campaign_id" value="'.Tools::safeOutput(Tools::getValue('pap_campaign_id', Configuration::get('PAP_CAMPAIGN_ID'))).'" />
            <p class="clear">'.$this->l('ID of a campaign you want to use for ALL sales generated in your store (leave empty to recognize campaign automatically)').'</p>
          </div>

          <label>'.$this->l('Commission approval ID').'</label>
          <div class="margin-form">
            <input type="text" name="pap_approval_id" value="'.Tools::safeOutput(Tools::getValue('pap_approval_id', Configuration::get('PAP_APPROVAL_ID'))).'" />
            <p class="clear">'.$this->l('ID of each payment status (see IDs in Presta Shop admin - Orders > Statuses) which should automatically approve a pending commission in Post Affiliate Pro after you receive a real payment. Comma separated if more than one. (supported since PAP/PAN version 5.3.20.1)').'</p>
          </div>

          <label>'.$this->l('Commission decline ID').'</label>
          <div class="margin-form">
            <input type="text" name="pap_decline_id" value="'.Tools::safeOutput(Tools::getValue('pap_decline_id', Configuration::get('PAP_DECLINE_ID'))).'" />
            <p class="clear">'.$this->l('ID of each payment status (see IDs in Presta Shop admin - Orders > Statuses) which should automatically decline a commission in Post Affiliate Pro. Comma separated if more than one. (supported since PAP/PAN version 5.3.20.1)').'</p>
          </div>
        </fieldset>

        <fieldset>
          <legend>'.$this->l('Extra data tracking').'</legend>';

          for ($i = 1; $i<=5; $i++) {
            $output .= '<label>'.$this->l('Extra data').' '.$i.'</label>
              <div class="margin-form">
                <select name="extra_data'.$i.'" style="color:#000;width:145px">
                  <option value="0" '.((Tools::getValue('extra_data'.$i, Configuration::get('EXTRA_DATA'.$i)) == '0')?'selected':'').'></option>
                  <option value="product_name" '.((Tools::getValue('extra_data'.$i, Configuration::get('EXTRA_DATA'.$i)) == 'product_name')?'selected':'').'>product name</option>
                  <option value="product_id" '.((Tools::getValue('extra_data'.$i, Configuration::get('EXTRA_DATA'.$i)) == 'product_id')?'selected':'').'>product ID</option>
                  <option value="product_quantity" '.((Tools::getValue('extra_data'.$i, Configuration::get('EXTRA_DATA'.$i)) == 'product_quantity')?'selected':'').'>product quantity</option>
                  <option value="product_price" '.((Tools::getValue('extra_data'.$i, Configuration::get('EXTRA_DATA'.$i)) == 'product_price')?'selected':'').'>product unit price</option>
                  <option value="product_reference" '.((Tools::getValue('extra_data'.$i, Configuration::get('EXTRA_DATA'.$i)) == 'product_reference')?'selected':'').'>product reference</option>
                  <option value="product_category" '.((Tools::getValue('extra_data'.$i, Configuration::get('EXTRA_DATA'.$i)) == 'product_category')?'selected':'').'>product category ID</option>
                  <option value="tax_rate" '.((Tools::getValue('extra_data'.$i, Configuration::get('EXTRA_DATA'.$i)) == 'tax_rate')?'selected':'').'>product tax rate</option>
                  <option value="id_shop" '.((Tools::getValue('extra_data'.$i, Configuration::get('EXTRA_DATA'.$i)) == 'id_shop')?'selected':'').'>product shop ID</option>
                  <option value="id_warehouse" '.((Tools::getValue('extra_data'.$i, Configuration::get('EXTRA_DATA'.$i)) == 'id_warehouse')?'selected':'').'>product warehouse ID</option>
                  <option value="customer_name" '.((Tools::getValue('extra_data'.$i, Configuration::get('EXTRA_DATA'.$i)) == 'customer_name')?'selected':'').'>customer name</option>
                  <option value="customer_email" '.((Tools::getValue('extra_data'.$i, Configuration::get('EXTRA_DATA'.$i)) == 'customer_email')?'selected':'').'>customer email</option>
                  <option value="customer_id" '.((Tools::getValue('extra_data'.$i, Configuration::get('EXTRA_DATA'.$i)) == 'customer_id')?'selected':'').'>customer ID</option>
                  <option value="customer_group" '.((Tools::getValue('extra_data'.$i, Configuration::get('EXTRA_DATA'.$i)) == 'customer_group')?'selected':'').'>customer group</option>
                </select>
              </div>';
          }
        $output .= '</fieldset>

        <fieldset>
          <legend>'.$this->l('Affiliate signup settings').'</legend>

          <label>'.$this->l('Register affiliates').'</label>
          <div class="margin-form">
            <input type="checkbox" name="register_affiliate" value="Y" '.((Tools::getValue('register_affiliate', Configuration::get('REGISTER_AFFILIATE')))?'checked="checked" ':'').'/>
            <p class="clear">'.$this->l('An affiliate account will be created automatically for customers.').'</p>
          </div>';

        if (Tools::getValue('register_affiliate', Configuration::get('REGISTER_AFFILIATE'))) {
            $output .= '<label>'.$this->l('Affiliate approval product ID').'</label>
              <div class="margin-form">
                <input type="text" name="affiliate_approval_prodid" value="'.Tools::safeOutput(Tools::getValue('affiliate_approval_prodid', Configuration::get('AFFILIATE_APPROVAL_PRODUCT_ID'))).'" />
                <p class="clear">'.$this->l('When a defined product is ordered, approve affiliate (created with the option above). Comma separated if more than one.').'</p>
              </div>';
        }

        $output .= '</fieldset>
        <center><input type="submit" name="submitPAP" value="Submit" style="background-color:#00aff0;color:#fff;text-transform:uppercase;padding:5px;border-width:0;border-radius:3px" /></center>
        </form>';

        return $output;
    }

  function hookFooter($params) {
      $pap_install_url = Configuration::get('PAP_INSTALL_URL');
      $account_id = Configuration::get('PAP_ACCOUNT_ID');
      $campaign_id = Configuration::get('PAP_CAMPAIGN_ID');

      $this->context->smarty->assign('pap_install_url', $pap_install_url);
      $this->context->smarty->assign('account_id', $account_id);
      $this->context->smarty->assign('campaign_id', $campaign_id);
      $this->context->smarty->assign('per_product', '');
      $this->context->smarty->assign('isOrder', false);

      return $this->display(__FILE__, 'footer.tpl');
  }
  
  function hookDisplayFooter($params) {
	  return $this->hookFooter($params);
  }
  
  function hookDisplayFooterBuilder() {
	  return $this->hookFooter($params);
  }

  function hookActionCustomerAccountAdd($params) {
      $register = Configuration::get('REGISTER_AFFILIATE');
      if (empty($register)) {
          $this->debug('Affiliate register is not enabled, ending.');
          return false;
      }

      $customer = $params['newCustomer'];
      $this->debug('New customer: '.$customer->email.' - we will try to create an affiliate account here.');

      require_once(_PS_MODULE_DIR_.$this->name.'/PapApi.class.php');
      
      $response = $this->getSession();
      if ($response['session'] === null) {
          $this->debug($response['message']);
          return false;
      }
      
      $affiliate = new Pap_Api_Affiliate($response['session']);
      $affiliate->setUsername($customer->email);
      $affiliate->setFirstname($customer->firstname);
      $affiliate->setLastname($customer->lastname);
      $affiliate->setPassword($params['_POST']['passwd']);
      $parentId = Tools::getValue('pap_parent');
      $cookieId = Tools::getValue('pap_cookie');
      if (!$parentId && isset($params['_POST']['pap_parent'])) {
          $parentId = $params['_POST']['pap_parent'];
      }
      if (!$cookieId && isset($params['_POST']['pap_cookie'])) {
          $cookieId = $params['_POST']['pap_cookie'];
      }
      if ($parentId != '') {
          $affiliate->setParentUserId($parentId);
      }
      if ($cookieId != '') {
          $affiliate->setVisitorId($cookieId);
      }

      try {
          if ($affiliate->add()) {
              $this->debug('Affiliate saved successfully');
              return true;
          }
          else {
              $this->debug('Cannot save affiliate: '.$affiliate->getMessage());
              return false;
          }
      }
      catch (Exception $e) {
          $this->debug('AccountAdd - Error while communicating with PAP: '.$e->getMessage());
          return false;
      }
  }

	public function hookDisplayCustomerAccountForm($params) {
        $register = Configuration::get('REGISTER_AFFILIATE');
        if (empty($register)) {
            $this->debug('Affiliate register is not enabled, ending.');
            return null;
        }

        $pap_install_url = Configuration::get('PAP_INSTALL_URL');
        $this->context->smarty->assign('pap_install_url', $pap_install_url);

        return $this->display(__FILE__, 'form.tpl');
	}

  function approveAffiliate($email, $session = null) {
      if ($session === null) {
          $response = $this->getSession();
          if ($response['session'] === null) {
              $this->debug($response['message']);
              return false;
          }
          
          $session = $response['session'];
      }

      $affiliate = new Pap_Api_Affiliate($session);
      $affiliate->setUsername($email);

      try {
          $affiliate->load();
          $affiliate->setStatus('A');
          $affiliate->save();
          return true;
      }
      catch (Exception $e) {
          $this->debug('AffiliateApproval: Error while communicating with PAP: '.$e->getMessage());
          return false;
      }
  }

  function hookActionOrderStatusUpdate($params) {
      $statusIds = Configuration::get('PAP_APPROVAL_ID');
      $declineStatusIds = Configuration::get('PAP_DECLINE_ID');
      if (empty($statusIds) && empty($declineStatusIds)) {
          $this->debug('No approval or decline IDs defined, ending.');
          return false;
      }
      $statusIds = explode(',',$statusIds);
      $declineStatusIds = explode(',',$declineStatusIds);

      if (!file_exists(_PS_MODULE_DIR_.$this->name.'/PapApi.class.php')) {
          $this->debug('Cannot include API file, ending.');
          return false;
      }

      if (in_array($params['newOrderStatus']->id,$statusIds)) { // for approval
          $status = 'A';
      }
      elseif (in_array($params['newOrderStatus']->id,$declineStatusIds)) { // for declining
          $status = 'D';
      }
      else { // status not in defined statuses... ending
          $this->debug('Status not in defined statuses, ending.');
          return false;
      }

      require(_PS_MODULE_DIR_.$this->name.'/PapApi.class.php');

      $response = $this->getSession();
      if ($response['session'] === null) {
          $this->debug($response['message']);
          return false;
      }
          
      $session = $response['session'];

      if ($status == 'A') {
          // get customer ID from the order
          $order = new Order($params['id_order']);
          // load the customer and get email
          $customer = new Customer((int)($order->id_customer));
          $approvalIds = explode(',', Configuration::get('AFFILIATE_APPROVAL_PRODUCT_ID'));
          $products = $order->getProducts();
          foreach ($products as $product) {
              if (in_array($product['product_id'], $approvalIds) && ($status == 'A')) {
                  $this->approveAffiliate($customer->email, $session);
                  break; // no more products needed to approve affiliate
              }
          }
      }

      $request = new Pap_Api_TransactionsGrid($session);
      $request->addFilter('orderid', Gpf_Data_Filter::LIKE, $params['id_order'].'(%');
      if ($status == 'A') {
        $request->addFilter('rstatus', Gpf_Data_Filter::EQUALS, 'P'); // load only pending for approval
      }
      if ($status == 'D') {
        $request->addFilter('rstatus', 'IN', 'A,P'); // load only pending for approval
      }

      try {
          $request->sendNow();
          $recordset = $request->getGrid()->getRecordset();
          $ids = array();
          foreach($recordset as $rec) {
              $ids[] = $rec->get('id');
          }
      }
      catch (Exception $e) {
          $this->debug('API error while searching for the order with postfix: '.$e->getMessage());
          return false;
      }

      if (empty($ids)) {
          $this->debug('No commissions found, let try unprocessed transactions here.');
          $transaction = new Pap_Api_Transaction($session);
          // check if PAP already uses the new function approveByOrderId
          if (!method_exists($transaction,'approveByOrderId')) {
              $this->debug('Your PAP version is old (you can update it), we cannot change status of unprocessed transactions with it. Ending');
              return true;
          }
          $transaction->setOrderId($params['id_order']);
          if ($status == 'A') { // for approval
              $this->debug('Order '.$params['id_order'].' will be approved after status '.$params['newOrderStatus']->id.'.');
              $transaction->approveByOrderId();
          }
          else { // for declining
              $this->debug('Order '.$params['id_order'].' will be declined after status '.$params['newOrderStatus']->id.'.');
              $transaction->declineByOrderId();
          }
          $this->debug('Unprocessed transactions changed. Ending');
          return true;
      }
      else {
          try {
              $this->debug('We will be changing status of IDs: '.print_r($ids,true));
              $request = new Gpf_Rpc_FormRequest('Pap_Merchants_Transaction_TransactionsForm', 'changeStatus', $session);
              $request->addParam('ids',new Gpf_Rpc_Array($ids));
              $request->addParam('status',$status);
              $request->sendNow();
              $this->debug('Status changed successfully. Ending');
              return true;
          }
          catch (Exception $e) {
              $this->debug('API error while status changing: '.$e->getMessage());
              return false;
          }
      }
  }

  function hookOrderConfirmation($params) {
      if (isset($params['order'])) { // v 1.7
          $order = $params['order'];
      } else {
          $order = $params['objOrder']; // v 1.6
      }

      if (Validate::isLoadedObject($order)) {
          $conversion_rate = 1;
          $currency = new Currency(intval($order->id_currency));
          $pap_currency = $currency->iso_code;
          if ($order->id_currency != Configuration::get('PS_CURRENCY_DEFAULT')) {
              $conversion_rate = floatval($currency->conversion_rate);
              $currency = new Currency(intval(Configuration::get('PS_CURRENCY_DEFAULT')));
              $pap_currency = $currency->iso_code;
          }

          // coupon
          $coupon = Db::getInstance()->getRow('
          SELECT c.code from `'._DB_PREFIX_.'cart_rule` AS c, `'._DB_PREFIX_.'order_cart_rule` AS o
          WHERE o.`id_order`='.$order->id.'
          AND o.`id_cart_rule`= c.`id_cart_rule`'
          );

          $orderInfo['id'] = $order->id;
          $discountValue = 0;
          if (isset($order->total_discounts_tax_excl)) {
            $discountValue = $order->total_discounts_tax_excl;
          }
          $orderInfo['subtotal'] = Tools::ps_round(floatval($order->total_products - $discountValue) / floatval($conversion_rate), 2);

          $statusIds = Configuration::get('PAP_APPROVAL_ID');
          $statusIds = explode(',',$statusIds);
          $orderInfo['status'] = (in_array($order->getCurrentState(), $statusIds)?'A':'P');

          // Product information
          $products = $order->getProducts();
          $allItems = '';
          $extraData = array();
          $approvalIds = explode(',',Configuration::get('AFFILIATE_APPROVAL_PRODUCT_ID'));

          // compute discount percent
          $orderTotal = Tools::ps_round(floatval($order->total_products) / floatval($conversion_rate), 2);
          $discountValue = Tools::ps_round(floatval($discountValue) / floatval($conversion_rate), 2);
          $discountPercent = $discountValue/$orderTotal; // e.g. 0.2 for 20%

          // get product settings
          $productSelector = Configuration::get('PER_PRODUCT');
          $items = array();
          foreach ($products AS $product) {
            if ($productSelector != '0') {
                $items[] = array(
                  'OrderId' => intval($order->id),
                  'Product' => addslashes($this->getProductInfo($product, $productSelector)),
                  'Total' => $this->getPrice($product, $conversion_rate, $discountPercent),
                  'Data1' => $this->resolveExtraData('1', $product),
                  'Data2' => $this->resolveExtraData('2', $product),
                  'Data3' => $this->resolveExtraData('3', $product),
                  'Data4' => $this->resolveExtraData('4', $product),
                  'Data5' => $this->resolveExtraData('5', $product)
                );

                if ($orderInfo['status'] == 'A' && in_array($product['product_id'],$approvalIds)) { // if order has been approved, approve the affiliate as well
                  $this->approveAffiliate($this->context->customer->email);
                }
            }

            $allItems .= $product['product_id'].', ';
            // resolve extra data
            $extraData['Data1'] = $this->resolveExtraData('1');
            $extraData['Data2'] = $this->resolveExtraData('2');
            $extraData['Data3'] = $this->resolveExtraData('3');
            $extraData['Data4'] = $this->resolveExtraData('4');
            $extraData['Data5'] = $this->resolveExtraData('5');
          }
          $allItems = rtrim($allItems, ', ');

          $pap_install_url = Configuration::get('PAP_INSTALL_URL');
          $account_id = Configuration::get('PAP_ACCOUNT_ID');

          $per_product = true;
          if (Configuration::get('PER_PRODUCT') == 0) {
              $per_product = false;
          }

          $campaign_id = Configuration::get('PAP_CAMPAIGN_ID');

          $this->context->smarty->assign('items', $items);
          $this->context->smarty->assign('allitems', $allItems);
          $this->context->smarty->assign('order', $orderInfo);
          $this->context->smarty->assign('pap_install_url', $pap_install_url);
          $this->context->smarty->assign('account_id', $account_id);
          $this->context->smarty->assign('per_product', $per_product);
          $this->context->smarty->assign('pap_currency', $pap_currency);
          $this->context->smarty->assign('extraData', $extraData);
          $this->context->smarty->assign('campaign_id', $campaign_id);

          $this->context->smarty->assign('isOrder', true);
          if (!empty($coupon['code'])) {
              $this->context->smarty->assign('coupon_code', $coupon['code']);
          }
          else {
              $this->context->smarty->assign('coupon_code', '');
          }

          $this->debug('Registering a commission for order '.$order->id);
          return $this->display(__FILE__, 'footer.tpl');
      }
  }
  
  function hookDisplayOrderConfirmation($params) {
	  return $this->hookOrderConfirmation($params);
  }

    private function resolveExtraData($dateIndex, $product = array()) {
        $data = Configuration::get('EXTRA_DATA'.$dateIndex);
        if (empty($product)) {
            switch ($data) {
                case 'customer_email': return $this->context->customer->email;
                case 'customer_name': return $this->context->customer->firstname.' '.$this->context->customer->lastname;
                case 'customer_id': return $this->context->customer->id;
                case 'customer_group': return $this->getPAPCustomerGroup();
                default: return '';
            }
        }
        switch ($data) {
            case 'product_name':
            case 'product_id':
            case 'product_quantity':
            case 'product_price':
            case 'product_reference':
            case 'tax_rate':
            case 'id_shop':
            case 'id_warehouse': return $product[$data];
            case 'product_category': return $this->getProductCategory($product['product_id']);
            case 'customer_email': return $this->context->customer->email;
            case 'customer_name': return $this->context->customer->firstname.' '.$this->context->customer->lastname;
            case 'customer_id': return $this->context->customer->id;
            case 'customer_group': return $this->getPAPCustomerGroup();
            default: return '';
        }
    }
    
    private function getPAPCustomerGroup() {
        $papCustomer = new Customer($this->context->customer->id);
        $groups = $papCustomer->getGroups();
        $custGroups = '';
        foreach ($groups as $group) {
            $papGroup = new Group($group);
            $groupName = reset($papGroup->name); // $papGroup->name returns array of group names in different languages
            $custGroups .= $groupName . ', '; 
        }
        $custGroups = substr($custGroups, 0, -2);
        if ($custGroups === false) {
            $custGroups = '';
        }
        return $custGroups;
    }

    private function getProductCategory($productId) {
        $product = new Product($productId);
        try {
            return $product->id_category_default;
        } catch (Exception $e) {
            return '';
        }
    }

    private function getProductInfo($product, $selector) {
        switch ($selector) {
            case '1': return $product['product_id'];
            case '2': return $product['product_name'];
            default: return $this->getProductCategory($product['product_id']);
        }
    }

  function getPrice($product, $cr, $discountPercent) {
    $productPrice = floatval($product['total_wt'])*(1-$discountPercent);
    $price = $productPrice / floatval($cr); // $product['product_quantity_discount']
    $price = $price / (1 + ($product['tax_rate'] / 100));
    //$price = (Tools::ps_round($price, 2)) * intval($product['product_quantity']);
    return Tools::ps_round($price, 2);
  }
}